<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\DB;
use Spatie\MediaLibrary\HasMedia\HasMedia;
use Spatie\MediaLibrary\HasMedia\HasMediaTrait;

class Invoice extends Model implements HasMedia
{

    use HasMediaTrait, SoftDeletes;
    protected $fillable = [
        'payment',
        'date',
        'client_id',
        'total',
        'paid',
        'due',
        'date',
        'note',
        'creator_id',
        'branche_id',
        'entery_at',
        'exit_at',
        'kilo_counter',
        'type_id',
        'status_id',
        'tax',
        'review_required',
        'review_status',
        'review_note',
        'car_id'
    ];
    protected $with = ['details'];

    public function details()
    {
        return $this->belongsToMany(Product::class, 'details', 'invoice_id', 'product_id')
            ->withPivot(['id', 'cost', 'price', 'qty', 'discount', 'profit', 'item_status', 'item_note']);
    }
    public function services()
    {
        return $this->hasMany(InvoiceService::class, 'invoice_id', 'id');
    }
    public function setEnteryAtAttribute($value)
    {
        $this->attributes['entery_at'] = Carbon::parse($value);
    }

    public function getCurrentTaskAttribute()
    {
        $orderTask = $this->orderTask()->where('status', 'معلق')->first();
        $title = "كل المهام مكتمله";
        if ($orderTask) {
            $title = optional($orderTask->task)->title;
        }
        return $title;
    }
    public function statusList()
    {
        return $this->belongsToMany(Status::class, 'invoice_status', 'invoice_id', 'status_id')
            ->withPivot('note', 'creator_id', 'created_at', 'creator_name');
    }

    public function currentStatus()
    {
        return $this->belongsTo(Status::class, 'status_id', 'id');
    }
    public function type()
    {
        return $this->belongsTo(Type::class, 'type_id', 'id');
    }
    public function car()
    {
        return $this->belongsTo(ClientCar::class, 'car_id', 'id');
    }
    public function setExitAtAttribute($value)
    {
        $this->attributes['exit_at'] = $value ? Carbon::parse($value) : null;
    }

    public function tasks()
    {
        return $this->belongsToMany(Mission::class, 'order_task', 'order_id', 'mission_id')
            ->withPivot('status', 'start', 'note', 'user_id');
    }

    public function orderTask()
    {
        return $this->hasMany(OrderTask::class, 'order_id', 'id');
    }
    public function client()
    {
        return $this->belongsTo(Client::class)->withTrashed();
    }
    public function creator()
    {
        return $this->belongsTo(User::class, 'creator_id', 'id');
    }

    public function branche()
    {
        return $this->belongsTo(Branche::class, 'branche_id', 'id');
    }

    public function transactions()
    {
        return $this->morphMany(Transaction::class, 'transactionable', 'model_type', 'model_id');
    }

    public function getTotalDueAttribute()
    {
        $total = $this->total;
        $paid = $this->total_paid;
        return floor($total - $paid);
    }

    public function getTotalPaidAttribute()
    {
        $total_paid = floor($this->transactions()->sum('value') + $this->paid);
        return abs($total_paid);
    }

    public function scopeFilter(Builder $builder)
    {
        $from = request('start_date');
        $to = request('end_date');
        $user_id = request('user_id');
        $client_id = request('client_id');
        $entery_at = request('entery_at');
        $exit_at = request('exit_at');
        $status = request('status');
        $type = request('type');
        $branche = request('branche_id') ?: auth()->user()->branche_id;
        if ($status) {
            $builder->where('status_id', $status);
        }
        if ($type) {
            $builder->where('type_id', $type);
        }
        if ($branche) {
            $builder->where('branche_id', $branche);
        }
        if ($client_id) {
            $builder->where('client_id', $client_id);
        }
        if ($user_id) {
            $builder->where('creator_id', $user_id);
        }
        if (!empty($entery_at)) {
            $builder->whereRaw("DATE(entery_at) = '{$entery_at}'");
        }
        if (!empty($exit_at)) {
            $builder->whereRaw("DATE(exit_at) = '{$exit_at}'");
        }
        if (!empty($from)) {
            $builder->whereRaw("DATE(date) >= '{$from}'");
        }
        if (!empty($to)) {
            $builder->whereRaw("DATE(date) <= '{$to}'");
        }
        return $builder->orderBy('id', 'DESC');
    }
    public function gettaxValueAttribute()
    {
        $tax = $this->tax / 100;
        $total = $this->total;
        $taxplusone = 1 + $tax;
        $orignalValue = $total / $taxplusone;
        $taxvalue = $orignalValue * $tax;
        return $taxvalue;
    }
    public function getTotalWithoutTaxAttribute()
    {
        $tax = $this->tax / 100;
        $total = $this->total;
        $taxplusone = 1 + $tax;
        $orignalValue = $total / $taxplusone;
        $taxvalue = $orignalValue * $tax;
        $value = $total - $taxvalue;
        return ceil($value);
    }
    public function getTotalAttribute($val)
    {
        return ceil($val);
    }
    public function getPaidAttribute($val)
    {
        return ceil($val);
    }
    public function getDueAttribute($val)
    {
        return ceil($val);
    }
    public function getPostDateAttribute()
    {
        $months = array("Jan" => "يناير", "Feb" => "فبراير", "Mar" => "مارس", "Apr" => "أبريل", "May" => "مايو", "Jun" => "يونيو", "Jul" => "يوليو", "Aug" => "أغسطس", "Sep" => "سبتمبر", "Oct" => "أكتوبر", "Nov" => "نوفمبر", "Dec" => "ديسمبر");
        //$your_date = $this->created_at->addDays(1)->format('Y-m-d');    //date('y-m-d'); // The Current Date
        if (!$this->date)
            return "";
        $your_date = $this->date;
        // if(app()->getLocale()=='en'){
        //     return date('l, d F Y',strtotime($your_date));
        // }
        $en_month = date("M", strtotime($your_date));
        foreach ($months as $en => $ar) {
            if ($en == $en_month) {
                $ar_month = $ar;
            }
        }
        $find = array("Sat", "Sun", "Mon", "Tue", "Wed", "Thu", "Fri");
        $replace = array("السبت", "الأحد", "الإثنين", "الثلاثاء", "الأربعاء", "الخميس", "الجمعة");
        $ar_day_format = date("D", strtotime($your_date)); //date('D'); // The Current Day
        $ar_day = str_replace($find, $replace, $ar_day_format);
        $day_num = date("d", strtotime($your_date));
        $year_num = date("Y", strtotime($your_date));
        header('Content-Type: text/html; charset=utf-8');
        $standard = array("0", "1", "2", "3", "4", "5", "6", "7", "8", "9");
        $eastern_arabic_symbols = array("٠", "١", "٢", "٣", "٤", "٥", "٦", "٧", "٨", "٩");
        $current_date = $ar_day . ', ' . $day_num . ' ' . $ar_month . ' ' . $year_num;
        $arabic_date = str_replace($standard, $eastern_arabic_symbols, $current_date);

        return $arabic_date;
    }

    public function getDateAr($your_date)
    {
        $months = array("Jan" => "يناير", "Feb" => "فبراير", "Mar" => "مارس", "Apr" => "أبريل", "May" => "مايو", "Jun" => "يونيو", "Jul" => "يوليو", "Aug" => "أغسطس", "Sep" => "سبتمبر", "Oct" => "أكتوبر", "Nov" => "نوفمبر", "Dec" => "ديسمبر");
        if (!$your_date)
            return "";
        $en_month = date("M", strtotime($your_date));
        $curTime = strtotime($your_date);
        $arrEn = array('am', 'pm');
        $arrAr = array('صباحاً', 'مساءً');
        $newTimeFormat = date("h:i ", $curTime) . str_replace($arrEn, $arrAr, date("a", $curTime));

        foreach ($months as $en => $ar) {
            if ($en == $en_month) {
                $ar_month = $ar;
            }
        }
        $find = array("Sat", "Sun", "Mon", "Tue", "Wed", "Thu", "Fri");
        $replace = array("السبت", "الأحد", "الإثنين", "الثلاثاء", "الأربعاء", "الخميس", "الجمعة");
        $ar_day_format = date("D", strtotime($your_date)); //date('D'); // The Current Day
        $ar_day = str_replace($find, $replace, $ar_day_format);
        $day_num = date("d", strtotime($your_date));
        $year_num = date("Y", strtotime($your_date));
        header('Content-Type: text/html; charset=utf-8');
        $standard = array("0", "1", "2", "3", "4", "5", "6", "7", "8", "9");
        $eastern_arabic_symbols = array("٠", "١", "٢", "٣", "٤", "٥", "٦", "٧", "٨", "٩");
        $current_date = $ar_day . ', ' . $day_num . ' ' . $ar_month . ' ' . $year_num;
        $arabic_date = str_replace($standard, $eastern_arabic_symbols, $current_date);

        return $arabic_date . " " . $newTimeFormat;
    }
    public function addInvoiceTask($order)
    {

        try {
            DB::beginTransaction();
            $tasksList = [];
            $missionList = [
                ['rolename' => 'مسئول الأستقبال', 'department_id' => 1, 'task_id' => 1, 'status' => 'قبول'],
                ['rolename' => 'مسئول الصيانة', 'department_id' => 2, 'task_id' => 2, 'status' => 'معلق'],
                ['rolename' => 'مسئول قطع الغيار', 'department_id' => 3, 'task_id' => 3, 'status' => 'معلق'],
            ];
            if ($order->review_required) {
                $missionList[] = ['rolename' => 'مسئول الضمان', 'department_id' => 4, 'task_id' => 4, 'status' => 'معلق'];
            }
            $missionList[] = ['rolename' => 'مسئول الأستقبال', 'department_id' => 1, 'task_id' => 5, 'status' => 'معلق'];
            foreach ($missionList as $row) {
                $emp = User::query();
                if (auth()->user()->branche_id) {
                    $emp->where('branche_id', auth()->user()->branche_id);
                }
                $emp = $emp->whereHas("roles", function ($q) use ($row) {
                    $q->where("name", $row['rolename']);
                })->first();
                if (!$emp) {
                    $emp = User::query()->whereHas("roles", function ($q) use ($row) {
                        $q->where("name", $row['rolename']);
                    })->first();
                }
                if (!$emp) {
                    throw new \Exception("يجب إضافة مستخدمين بالصلاحيات التالية " . $row['rolename']);
                }
                $employeeId = $emp->id;

                $tasksList[] = [
                    'branche_id' => auth()->user()->branche_id,
                    'must_completed' => 1,
                    'order_id' => $order->id,
                    'client_id' => $order->client_id,
                    'department_id' => $row['department_id'],
                    'mission_id' => $row['task_id'],
                    'user_id' => $employeeId,
                    'status' => $row['status'],
                    'notifcation_date' => date('Y-m-d'),
                ];
            }
            if ($tasksList) {
                OrderTask::query()->insert($tasksList);
            }
            DB::commit();
        } catch (\Exception $e) {
            DB::rollBack();
            dd($e->getMessage());
        }
    }
}